"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const admin = require("firebase-admin");
const documentService = require("./document.service");
const systemNotificationService = require("./system-message.service");
const siteCollection = () => admin.firestore().collection('tanam').doc(process.env.GCLOUD_PROJECT);
function queryPageContext(documentTypeId, queryOpts = {}) {
    return __awaiter(this, void 0, void 0, function* () {
        console.log(`[queryPageContext] ${documentTypeId}, query=${JSON.stringify(queryOpts)}`);
        const orderByField = queryOpts.orderBy && queryOpts.orderBy.field || 'updated';
        const sortOrder = queryOpts.orderBy && queryOpts.orderBy.sortOrder || 'desc';
        const limit = queryOpts.limit || 20;
        console.log(`[queryPageContext] effective query ${JSON.stringify({ orderByField, sortOrder, limit })}`);
        let querySnap;
        try {
            querySnap = yield siteCollection()
                .collection('documents')
                .where('status', '==', 'published')
                .where('documentType', '==', documentTypeId)
                .orderBy(orderByField, sortOrder)
                .limit(limit)
                .get();
        }
        catch (err) {
            console.error(`[queryPageContext] ${JSON.stringify(err)}`);
            const details = err.details;
            if (details.indexOf('firestore/indexes?create')) {
                yield systemNotificationService.reportMissingIndex(details);
            }
            else {
                yield systemNotificationService.reportUnknownError(details);
            }
            return [];
        }
        console.log(`[queryPageContext] num results: ${querySnap.docs.length}`);
        const result = [];
        for (const doc of querySnap.docs) {
            console.log(`[queryPageContext] ${JSON.stringify(doc.data())}`);
            result.push(_toContext(doc.data()));
        }
        return result;
    });
}
exports.queryPageContext = queryPageContext;
function getPageContextById(docId) {
    return __awaiter(this, void 0, void 0, function* () {
        console.log(`[getPageContextById] ${JSON.stringify(docId)}`);
        const doc = yield documentService.getDocumentById(docId);
        return !!doc ? _toContext(doc) : null;
    });
}
exports.getPageContextById = getPageContextById;
function getPageContextByUrl(url) {
    return __awaiter(this, void 0, void 0, function* () {
        console.log(`[getPageContextByUrl] ${JSON.stringify(url)}`);
        const documents = yield documentService.getDocumentByUrl(url || '/');
        console.log(`[getPageContextByUrl] Number of query results: ${documents.length}`);
        return documents.length === 0 ? null : _toContext(documents[0]);
    });
}
exports.getPageContextByUrl = getPageContextByUrl;
function _toContext(document) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!document) {
            return null;
        }
        const siteInfo = (yield siteCollection().get()).data();
        const siteContext = {
            domain: siteInfo.primaryDomain,
            analytics: siteInfo.analytics,
            url: `https://${siteInfo.primaryDomain}`,
            theme: siteInfo.theme,
            title: siteInfo.title,
        };
        const documentContext = {
            id: document.id,
            documentType: document.documentType,
            data: _normalizeData(document.data),
            title: document.title,
            standalone: document.standalone,
            url: document.standalone ? document.url : null,
            permalink: document.standalone ? `${siteContext.url}${document.url}` : null,
            revision: document.revision,
            status: document.status,
            tags: document.tags,
            created: document.created.toDate(),
            updated: document.updated.toDate(),
            published: !!document.published
                ? document.published.toDate()
                : null,
        };
        return {
            document: documentContext,
            site: siteContext,
        };
    });
}
function _normalizeData(data) {
    for (const key in data) {
        const val = data[key];
        if (val && val.toDate) {
            // Applies to Firestore timestamps
            data[key] = val.toDate();
        }
    }
    return data;
}
//# sourceMappingURL=page-context.service.js.map