"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const functions = require("firebase-functions");
const https = require("https");
exports.cacheTask = functions.database.ref('tanam/{siteId}/tasks/cache/{taskId}').onCreate((snap) => __awaiter(this, void 0, void 0, function* () {
    const task = snap.val();
    const promises = [snap.ref.remove()];
    switch (task.action) {
        case 'delete':
            promises.push(makeRequest('PURGE', task.domain, task.url));
            break;
        case 'create':
            promises.push(makeRequest('GET', task.domain, task.url));
            break;
        case 'update':
        default:
            yield makeRequest('PURGE', task.domain, task.url);
            promises.push(makeRequest('GET', task.domain, task.url));
    }
    return promises;
}));
/**
 * Send a request to either purge or heat the CDN
 *
 * For `GET` requests, this function will pre-heat the CDN cache by trying to request the updated document
 * before any visitor is hitting it. It's a race condition agains visitors to try and beat them to be the
 * first to trigger a new render. The hope is that this function will be the only client that is exposed to
 * the relativly slow operation of rendering a document. Any subsequest client request will get a static
 * version from the CDN cache.
 *
 * The `PURGE` action removes the previously cached document at given URL. This allows for the next `GET`
 * request to get a freshly rendered document that accurately represents the current content.
 *
 * @param action PURGE | GET
 * @param host Domain host (e.g. somesubdomain.example.com)
 * @param path Path part of the URL
 */
function makeRequest(action, host, path) {
    const normalizedPath = path.replace(/\/+/g, '/');
    const opts = {
        hostname: host,
        port: 443,
        path: normalizedPath,
        method: action,
    };
    console.log(`[makeRequest] BEGIN ${JSON.stringify(opts)}`);
    return new Promise((resolve) => {
        https.get(opts, (res) => {
            console.log(`[makeRequest] FINISHED ${JSON.stringify(Object.assign({}, opts, { statusCode: res.statusCode, statusMessage: res.statusMessage }))}`);
            resolve(null);
        });
    });
}
//# sourceMappingURL=cache.js.map