"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const admin = require("firebase-admin");
const functions = require("firebase-functions");
const sharp = require("sharp");
exports.processImageUpload = functions.storage.object().onFinalize((object) => __awaiter(this, void 0, void 0, function* () {
    if (!object.name.startsWith(`tanam/${process.env.GCLOUD_PROJECT}/upload/`)) {
        console.log(`Not an upload task: ${object.name} (${object.contentType})`);
        return null;
    }
    if (!object.contentType.startsWith('image/')) {
        console.log(`File is not an image: ${object.name} (${object.contentType})`);
        return null;
    }
    console.log(`Processing file: ${object.name}`);
    const bucket = admin.storage().bucket(object.bucket);
    const [originalFileBuffer] = yield bucket.file(object.name).download();
    const resizeAndConvertImage = (size) => sharp(originalFileBuffer)
        .resize(size, size, {
        withoutEnlargement: true,
        fit: sharp.fit.inside,
    })
        .toFormat(sharp.format.webp)
        .toBuffer();
    const originalSuffix = object.name.lastIndexOf('.') > 0 ? object.name.substr(object.name.lastIndexOf('.')) : '';
    const firestoreRef = admin.firestore()
        .collection('tanam').doc(process.env.GCLOUD_PROJECT)
        .collection('files').doc();
    const newFileName = firestoreRef.id;
    const newFilePath = `tanam/${process.env.GCLOUD_PROJECT}/images/`;
    const metadata = {
        contentType: 'image/webp',
        metadata: object.metadata,
    };
    const tanamFile = {
        id: firestoreRef.id,
        title: object.name.substr(object.name.lastIndexOf('/') + 1),
        bucket: object.bucket,
        filePath: [newFilePath, newFileName, originalSuffix].join(''),
        updated: admin.firestore.FieldValue.serverTimestamp(),
        created: admin.firestore.FieldValue.serverTimestamp(),
        bytes: originalFileBuffer.byteLength,
        variants: {
            small: `${newFilePath}${newFileName}_small.webp`,
            medium: `${newFilePath}${newFileName}_medium.webp`,
            large: `${newFilePath}${newFileName}_large.webp`,
        },
        mimeType: object.contentType,
        fileType: 'image',
    };
    return yield Promise.all([
        firestoreRef.set(tanamFile),
        bucket.file(object.name).delete(),
        bucket.file(tanamFile.filePath).save(originalFileBuffer, object.metadata),
        bucket.file(tanamFile.variants.small).save(yield resizeAndConvertImage(300), metadata),
        bucket.file(tanamFile.variants.medium).save(yield resizeAndConvertImage(800), metadata),
        bucket.file(tanamFile.variants.large).save(yield resizeAndConvertImage(1600), metadata),
    ]);
}));
exports.uploadAssetFiles = functions.storage.object().onFinalize((object) => __awaiter(this, void 0, void 0, function* () {
    if (!object.name.startsWith(`tanam/${process.env.GCLOUD_PROJECT}/themes/`)) {
        console.log(`Not an upload asset file task: ${object.name} (${object.contentType})`);
        return null;
    }
    console.log('[UploadAssetFiles]' + JSON.stringify(object));
    const objectNameArr = object.name.split('/');
    const themeId = objectNameArr[3];
    const originalSuffix = object.name.lastIndexOf('.') > 0 ? object.name.substr(object.name.lastIndexOf('.')) : '';
    const firestoreRef = admin.firestore()
        .collection('tanam').doc(process.env.GCLOUD_PROJECT)
        .collection('themes').doc(themeId)
        .collection('assets').doc();
    const tanamFile = {
        id: firestoreRef.id,
        title: object.name.substr(object.name.lastIndexOf('/') + 1),
        bucket: object.bucket,
        filePath: object.name,
        updated: admin.firestore.FieldValue.serverTimestamp(),
        created: admin.firestore.FieldValue.serverTimestamp(),
        bytes: Number(object.size),
        mimeType: object.contentType,
        fileType: object.contentType,
    };
    return firestoreRef.set(tanamFile);
}));
//# sourceMappingURL=storage.js.map